<script type="text/javascript" src='{{ STATIC_URL }}js/testrun_actions.js'></script>
<script type="text/javascript" src='{{ STATIC_URL }}js/management_actions.js'></script>
<script type="text/javascript">
	Nitrate.Utils.after_page_load(Nitrate.TestRuns.List.on_load);
</script>
{% include "run/common/run_advance_filtered.html" %}
