<form id="cases_form" action="{% url "cases-clone" %}" method="post">
	<div id="contentTab" class="mixbar">
		{#}<span class="tit">{{ test_cases.count }} Cases</span>{#}
		<span>
			{% if perms.testcases.add_testcase %}
			<input type="submit" value="Clone" title="clone selected test plans" />
			{% endif %}
			<input type="button" disabled="disabled"
                   id="case_list_printable" value="Printable copy"
                   title="Create the printable copy for selected cases." />
			<input type="button" id="export_selected_cases" value="Export" title="Export the cases for selected cases." />
		</span>
	</div>
	<table class="list border-bottom" id="testcases_table" cellpadding="0" cellspacing="0" border="0">
		<thead>
			<tr>
				<th width="18px">
					<a id="id_blind_all_link">
						<img src="{{ STATIC_URL }}images/t1.gif" border="0" alt="" class="collapse-all">
					</a>
				</th>
				<th width="20" class="nosort">
					<input type="checkbox" id="id_checkbox_all_case" title="Select all/Select none">
				</th>
				<th class="widthID">ID</th>
				<th>Summary</th>
				<th width="100">Author</th>
				<th width="130">Default Tester</th>
				<th width="160">Automated</th>
				<th width="90" >Status</th>
				<th width="120">Category</th>
				<th width="60">Priority</th>
				<th width="140">Created</th>
			</tr>
		</thead>
		<tbody>
		</tbody>
	</table>
</form>
