<table class="list"  class="list" cellpadding="0" cellspacing="0" border="0">
	<tbody>
		<tr >
			<td colspan="3"><span class="system_help">Change Log of removing test case.</span></td>
		</tr>
		{% for log in test_plan.log %}
		<tr class="{% cycle 'even' 'odd'%}">
			<td width="15%">{{ log.date }}</td>
			<td width="8%">{{ log.who }}</td>
			<td width="7%">{{ log.field }}</td>
            <td width="35%">{{ log.original_value }}</td>
            <td width="35%">{{ log.new_value }}</td>
		</tr>
		{% endfor %}
	</tbody>
</table>
