<table id="testruns_table" class="list border-bottom" cellpadding="0" cellspacing="0" border="0">
	<thead>
		<tr >
			<th class="nosort" width="30"><input id='id_check_all_runs' type="checkbox" title="Select all/Select none" /></th>
			<th class="number nosort widthID">ID</th>
			<th class="text nosort">Summary</th>
			<th class="text nosort" width="80">Manager</th>
			<th class="text nosort" width="100">Default Tester</th>
			<th class="text nosort" style="display:none" id="col_plan_head" >Plan</th>
			<th class="text nosort" width="130">Product</th>
			<th class="sortable nosort" width="120">Product Version</th>
			<th class="text nosort" width="100" title='Plan Environment Group'>Environment</th>
			<th class="number nosort" width="40">Cases</th>
			<th class="text nosort" width="80">Status</th>
			<th class="number nosort" width="110">Completed</th>
		</tr>
	</thead>
	<tbody>
	</tbody>
</table>
