<div class="mixbar">
	<div class="marginLeft fixed">
		<input id="js-change-parent-node" class="edit_node icon_plan left_float" type="button" value="Change parent node for plan {{ test_plan.pk }}" />
		<input id="js-add-child-node" class="add_node icon_plan left_float" type="button" value="Add child node to current plan {{ test_plan.pk }}" />
		<input id="js-remove-child-node" class="remove_node icon_plan left_float" {% if not test_plan.child_set.exists %}disabled{% endif %} type="button" value="Remove child node from current plan {{ test_plan.pk }}" />
	</div>
	<div class="treeview_toolbar_text">
		Only displayed following items in the tree
		<ul>
			<li>Current plan</li>
			<li>One level parent plan</li>
			<li>The plans with same parent</li>
			<li>One level children plans</li>
		</ul>
	</div>
</div>
<div id="id_tree_container" data-param="{{ test_plan.pk }}">
	<div class="ajax_loading"></div>
</div>